#ifndef _DS_ILOCK_H
#define _DS_ILOCK_H

#include <dslib.h>

#if defined(HAVE_PTHREAD_H)
# include <pthread.h>
#elif defined(HAVE_WIN32_THREADS)
# include <windows.h>
#else /* defined(HAVE_PTHREAD_H) */
# error "Missing threading library!"
#endif /* defined(HAVE_PTHREAD_H) */

#include <dsmalloc.h>


/*
OPCItem Locking facility. Should be based on multiple read one write scheme.

Linux Implementation Notes (glibc 2.3.2):
1. EDEADLK isn't returned when trying occure lock from thread alredy have it
2. Lock locked from one thread can be unlocked from another one

This implementation provides partly support for EDEADLK returning. Partly means 
what in cases we have alredy occured rdlock and now trying get wrlock, - no 
error will be returned and deadlock will occure.
*/


struct DSILockContextT {
    DSMallocContext malloc;
#if defined(HAVE_PTHREAD_H)
# ifdef DS_SUPPORT_RWLOCK
    pthread_rwlock_t lock_mutex;
    pthread_rwlock_t pid_mutex;
# else
    pthread_mutex_t lock_mutex;
    pthread_mutex_t pid_mutex;
# endif /* DS_SUPPORT_RWLOCK */
    pthread_t pid;
#else /* defined(HAVE_WIN32_THREADS) */
    HANDLE lock_mutex;
    HANDLE pid_mutex;
    DWORD pid;
#endif /* defined(HAVE_PTHREAD_H) */
    unsigned char initialized;
};
typedef struct DSILockContextT *DSILockContextS;
typedef struct DSILockContextT *DSILockContext;

DSILockContext dsILockCreateContext(DSMallocContext mctx);
int dsILockInitContext(DSILockContext ctx);
void dsILockFreeContext(DSILockContext ctx);
void dsILockDestroyContext(DSILockContext ctx);
int dsILockRead(DSILockContext ctx);
int dsILockWrite(DSILockContext ctx);
int dsILockTryRead(DSILockContext ctx);
int dsILockTryWrite(DSILockContext ctx);
int dsILockReturn(DSILockContext ctx);

#endif /* _DS_ILOCK_H */
