#ifndef _OPC_DATA_MANAGER_H
#define _OPC_DATA_MANAGER_H

#define OPC_REPRESENTATION_NATIVE 0
#define OPC_REPRESENTATION_OPCXML 1

#include <dslib.h>
#include <opcdatachunk.h>
#include <dsallocator.h>
#include <opcuserdata.h>

typedef struct OPCDataManagerContextT *OPCDataManagerContext;

typedef unsigned long (*OPCDataManagerGetIdFunction)(OPCDataManagerContext ctx, const char *repname);
typedef int (*OPCDataManagerAcquireFunction)(OPCDataManagerContext ctx);
typedef int (*OPCDataManagerPushDataFunction)(OPCDataManagerContext ctx, unsigned long repid, OPCDataInfo info, const void *data, unsigned long size);
typedef int (*OPCDataManagerPushAllocationFunction)(OPCDataManagerContext ctx, unsigned long repid, DSAllocatorContext allocator, OPCDataInfo info, DSAllocation allocation);

typedef DSAllocatorContext (*OPCDataManagerGetAllocatorFunction)(OPCDataManagerContext ctx, unsigned long repid, OPCDataInfo info);
typedef void (*OPCDataManagerReturnAllocatorFunction)(OPCDataManagerContext ctx, unsigned long repid, DSAllocatorContext allocator);
typedef void (*OPCDataManagerReturnDataFunction)(OPCDataManagerContext ctx, OPCDataInfo info);

typedef int (*OPCDataManagerGetDataFunction)(OPCDataManagerContext ctx, unsigned long repid, OPCDataInfo *info, OPCRepresentationData *data);
/*ToDo*/typedef int (*OPCDataManagerGetDataFirstFunction)(OPCDataManagerContext ctx, unsigned long repid, OPCDataInfo *info, OPCRepresentationData *data);
/*ToDo*/typedef int (*OPCDataManagerGetDataNextFunction)(OPCDataManagerContext ctx, unsigned long repid, OPCDataInfo *info, OPCRepresentationData *data);

typedef int (*OPCDataManagerRefFunction)(OPCDataManagerContext ctx);
typedef void (*OPCDataManagerUnrefFunction)(OPCDataManagerContext ctx);

/*repid if requierd!
typedef int (*OPCDataManagerPrepareDataFunction)(OPCDataManagerContext ctx, OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data);
typedef void (*OPCDataManagerDestroyDataFunction)(OPCDataManagerContext ctx, OPCUserData user_data);
typedef int (*OPCDataManagerSendDataFunction)(OPCDataManagerContext ctx, OPCUserData user_data, DSSendFunction function, void *attr);
*/


typedef void (*OPCDataManagerMantainFunction)(OPCDataManagerContext ctx);


struct OPCDataManagerContextT {
    OPCDataManagerGetIdFunction GetID;
    OPCDataManagerAcquireFunction Acquire;
    OPCDataManagerPushDataFunction PushData;
    OPCDataManagerPushAllocationFunction PushAllocation;
    OPCDataManagerGetAllocatorFunction GetAllocator;
    OPCDataManagerReturnAllocatorFunction ReturnAllocator;
    OPCDataManagerGetDataFunction GetData;
    OPCDataManagerGetDataFirstFunction GetFirst;
    OPCDataManagerGetDataNextFunction GetNext;
    OPCDataManagerReturnDataFunction ReturnData;
    OPCDataManagerRefFunction Ref;
    OPCDataManagerUnrefFunction Unref;
/*    OPCDataManagerPrepareDataFunction PrepareUserData;
    OPCDataManagerDestroyDataFunction DestroyUserData;
    OPCDataManagerSendDataFunction SendData;*/
    OPCDataManagerMantainFunction Mantain;
};

#define opcDataManagerGetId(ctx, repname) \
    ((ctx&&ctx->GetID)?(ctx->GetID(ctx, repname)):((unsigned long)-1))
#define opcDataManagerAcquire(ctx) \
    ((ctx&&ctx->Acquire)?(ctx->Acquire(ctx)):DS_ERR_NOTSUPPORTED)
#define opcDataManagerPushData(ctx,repid,info,data,size) \
    ((ctx&&ctx->PushData)?(ctx->PushData(ctx,repid,info,data,size)):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerPushAllocation(ctx,repid,allocator,info,allocation) \
    ((ctx&&ctx->PushAllocation)?(ctx->PushAllocation(ctx,repid,allocator,info,allocation)):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerGetAllocator(ctx,repid,info) \
    ((ctx&&ctx->GetAllocator)?(ctx->GetAllocator(ctx,repid,info)):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerReturnAllocator(ctx,repid,allocator) \
    ((ctx&&ctx->ReturnAllocator)?(ctx->ReturnAllocator(ctx,repid,allocator)):0)
#define opcDataManagerGetData(ctx,repid,info,data) \
    ((ctx&&ctx->GetData)?(ctx->GetData(ctx,repid,info,data)):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerReturnData(ctx,info) \
    ((ctx&&ctx->ReturnData)?ctx->ReturnData(ctx,info):0)
/*
#define opcDataManagerPrepareUserData(ctx,rep,options,data) \
    ((ctx->PrepareUserData)?(ctx->PrepareUserData(ctx,rep,options,data)):(opcDataManagerDirectUserData(rep,data)))
#define opcDataManagerDestroyUserData(ctx,data) \
    if (ctx->DestroyUserData) ctx->DestroyUserData(ctx,rep)
#define opcDataManagerSendData(ctx,data,func,attr) \
    ((ctx->SendData)?(ctx->SendData(ctx,data,func,attr)):OPC_ERR_NOTIMPLEMENTED
*/

#define opcDataManagerRef(ctx) \
    ((ctx&&ctx->Ref)?ctx->Ref(ctx):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerUnref(ctx) \
    ((ctx&&ctx->Unref)?ctx->Unref(ctx):DS_ERR_NOTIMPLEMENTED)
#define opcDataManagerMantain(ctx) \
    ((ctx&&ctx->Mantain)?ctx->Mantain(ctx):0)

int opcAcquireNotSupported(OPCDataManagerContext ctx);
int opcAcquireNotApplicable(OPCDataManagerContext ctx);

#endif /* _OPC_DATA_MANAGER_H */
