#include <dslib.h>
#include <dsassert.h>
#include <dsdatabuffer.h>
#include <dsallocator.h>
#include <opcdatafast.h>
#include <opcformatopcxml.h>

#include <opcuserdata.h>

int opcUserDataSendAllocation(OPCUserData user_data, DSSendFunction func, void *attr) {
    ds_assert_param((user_data)&&(user_data->info));
    
    return dsAllocatorCall(((DSAllocatorContext)user_data->info->ctx), ((DSAllocation)user_data->data), func, attr, 1);
}

int opcUserDataSendDataBuffer(OPCUserData user_data, DSSendFunction func, void *attr) {
    ds_assert_param(user_data);
    
    return dsDataBufferSend(((OPCUserData)user_data)->data, func, attr);
}


static void opcUserDataDestroyDefault(OPCUserData user_data) {
    if (user_data) {
	if (user_data->info) dsFree(user_data->info->malloc, user_data);
	else dsFree(NULL, user_data);
    }
}

int opcUserDataGetDefault(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    OPCUserDataAllocation data;
    
    if ((!user_data)||(!rep)||(!rep->info)) return DS_ERR_INVALID_ARGUMENT;
    
    data = (OPCUserDataAllocation)dsMalloc(rep->info->malloc, sizeof(OPCUserDataAllocationStr));
    memcpy(&data,rep,sizeof(struct OPCDataT));
    data->user_data.info = &data->user_info;
    
    data->user_info.ctx = rep->info->ctx;
    data->user_info.Destroy = opcUserDataDestroyDefault;
    data->user_info.malloc = rep->info->malloc;
    
    switch(rep->type) {
	case OPC_DATA_TYPE_DATABUFFER:
	    data->user_info.Send = opcUserDataSendDataBuffer;
	break;
	case OPC_DATA_TYPE_ALLOCATION:
	    data->user_info.Send = opcUserDataSendAllocation;
	break;
	case OPC_DATA_TYPE_XMLNODEPTR:
	default:
	    data->user_info.Send = NULL;
    }

    return 0;
}

static void opcUserDataDestroyOPCXML(OPCUserData user_data) {
    ds_assert_return(user_data);

    opcFormatOPCXMLFree((OPCFormatOPCXMLContext)user_data->info->ctx,NULL,NULL,(OPCData)user_data);
    free(user_data);
}

static void opcUserDataDestroyFree(OPCUserData user_data) {
    ds_assert_return(user_data);
    
    free(user_data);
}


int opcUserDataFastGetOPCXML(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    int err;
    OPCDataFastInfo repinfo;
    OPCType type;
    OPCFormatOPCXMLContext format;
    OPCUserInfo info;
    OPCUserDataAllocation data;

    if ((!user_data)||(!options)||(!rep)) return DS_ERR_INVALID_ARGUMENT;
    if (rep->type!=OPC_DATA_TYPE_DATABUFFER) return DS_ERR_INVALID_ARGUMENT;
    repinfo = (OPCDataFastInfo)rep->info;
    if (!repinfo) return DS_ERR_INVALID_ARGUMENT;
    if ((!repinfo->formats)||(!repinfo->datafast)) return DS_ERR_INVALID_ARGUMENT;
    format=(OPCFormatOPCXMLContext)repinfo->formats->opcxml_format;
    type=repinfo->datafast->type;
    if ((!format)||(!type)) return DS_ERR_INVALID_ARGUMENT;
    
    data = (OPCUserDataAllocation)malloc(sizeof(OPCUserDataAllocationStr));
    data->user_data.info = &data->user_info;
    info = data->user_data.info;

    info->ctx = (void*)format;
    info->malloc = NULL;
    info->Send = NULL;
    
    if (options->flags&OPC_REQUEST_OPTIONS_FLAG_XMLNODE_BEAR)
        info->Destroy = opcUserDataDestroyFree;
    else
        info->Destroy = opcUserDataDestroyOPCXML;

    err=opcFormatOPCXMLReadNative(format, type, rep->info->ctx, (OPCData)data, (OPCData)rep);
    if (err) free(data);
    else *user_data = (OPCUserData)data;

    return err;
}


int opcUserDataGetSelf(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    if (!user_data) return DS_ERR_INVALID_ARGUMENT;
    *user_data = (OPCUserData)rep;
    return 0;
}

